//$Id: CustomFormAuthenticator.java,v 1.18 2010/07/28 06:38:15 suresh Exp $
/*
 * Copyright 1999,2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package org.apache.catalina.authenticator;


import java.io.IOException;
import java.io.PrintWriter;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Logger;
import java.util.logging.Level;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jcifs.UniAddress;
import jcifs.http.NtlmSsp;
import jcifs.ntlmssp.Type1Message;
import jcifs.ntlmssp.Type2Message;
import jcifs.ntlmssp.Type3Message;
import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.SmbAuthException;
import jcifs.smb.SmbSession;
import jcifs.util.Base64;

import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.Response;

import org.apache.catalina.Realm;
import org.apache.catalina.Session;
import org.apache.catalina.deploy.LoginConfig;
import org.apache.tomcat.util.buf.CharChunk;
import org.apache.tomcat.util.buf.MessageBytes;

import com.adventnet.authentication.ACTIVEDIRECTORYINFO;
import com.adventnet.authentication.PAM;
import com.adventnet.authentication.twofactor.TwoFactorAuth;
import com.adventnet.authentication.util.AuthUtil;
import com.adventnet.ds.query.Column;
import com.adventnet.ds.query.Criteria;
import com.adventnet.ds.query.QueryConstants;
import com.adventnet.ds.query.SelectQuery;
import com.adventnet.ds.query.SelectQueryImpl;
import com.adventnet.ds.query.Table;
import com.adventnet.mfw.bean.BeanUtil;
import com.adventnet.persistence.DataAccess;
import com.adventnet.persistence.DataObject;
import com.adventnet.persistence.Persistence;
import com.adventnet.persistence.Row;
import com.adventnet.tools.prevalent.*;



/**
 * An <b>Authenticator</b> and <b>Valve</b> implementation of FORM BASED
 * Authentication, as described in the Servlet API Specification, Version 2.2.
 *
 * @author Craig R. McClanahan
 * @author Remy Maucherat
 * @version $Revision: 1.18 $ $Date: 2010/07/28 06:38:15 $
 */

public class CustomFormAuthenticator
extends FormAuthenticator {
	private static Logger logger = Logger.getLogger(CustomFormAuthenticator.class.getName());



	// ----------------------------------------------------- Instance Variables


	/**
	 * Descriptive information about this implementation.
	 */
	public static final String INFO ="org.apache.catalina.authenticator.FormAuthenticator/1.0";


	// ------------------------------------------------------------- Properties


	/**
	 * Return descriptive information about this Valve implementation.
	 */
	public String getInfo() {

		return (INFO);

	}


	// --------------------------------------------------------- Public Methods


	/**
	 * Authenticate the user making this request, based on the specified
	 * login configuration.  Return <code>true</code> if any specified
	 * constraint has been satisfied, or <code>false</code> if we have
	 * created a response challenge already.
	 *
	 * @param request Request we are processing
	 * @param response Response we are creating
	 * @param config    Login configuration describing how authentication
	 *              should be performed
	 *
	 * @exception IOException if an input/output error occurs
	 */
	public boolean authenticate(Request request,
			Response response,
			LoginConfig config)
		throws IOException {

			// References to objects we will need later
			HttpServletRequest hreq =
				(HttpServletRequest) request.getRequest();
			HttpServletResponse hres =
				(HttpServletResponse) response.getResponse();

			hreq.getSession().setAttribute("LoginConfig", config);
			Session session = null;
			boolean principalCreated=false;
			Principal principal = hreq.getUserPrincipal();
			Principal twoFactorNtlmCachedPrincipal=null;
			String ssoId = (String) request.getNote(Constants.REQ_SSOID_NOTE);
			if(principal==null && hreq.getSession().getAttribute("2FactorPrincipal")!=null)
			{
				twoFactorNtlmCachedPrincipal=(Principal)hreq.getSession().getAttribute("2FactorPrincipal");

				if(! validateTwoFactorAuth(hreq.getParameter("j_username"),hreq))
				{
					return false;
				}
				principalCreated=true;


			}
			if (principal == null)
			{
				if (System.getProperty("CheckLicenseInClient", "false").equals("true"))
				{
					String productName = System.getProperty("ProductName");
					String licenseFile = System.getProperty("LicenseValidator");

					// Licensing specific code starts here
					Wield w = Wield.getInstance();
					System.setProperty("WEB_LICENSE_CHECK", "true");

					String homeDir = System.getProperty("server.dir");
					w.validateInvoke(productName, homeDir, false, "lib", true);
					logger.info("The w.isBare() value :::::: " + w.isBare());
					if (!w.isBare())
					{
						// Show the License Registration page....
						RequestDispatcher disp = context.getServletContext().getRequestDispatcher(licenseFile);
						try
						{
							disp.forward(hreq, hres);
							response.finishResponse();
						}
						catch (Throwable t)
						{
							logger.log(Level.WARNING,"Unexpected error forwarding to error page", t);
						}
						return false;
					}
				}

			}

			// Have we already authenticated someone?
			if (principal != null)
			{
				logger.log(Level.FINER,"Already authenticated '" + principal.getName() + "'");
				// Associate the session with any existing SSO session
				if (ssoId != null) {
					associate(ssoId, request.getSessionInternal( true));
					hreq.getSession().setAttribute("JSESSIONIDSSO", ssoId);
				}
				return (true);
			}

			// Is there an SSO session against which we can try to reauthenticate?
			if (ssoId != null) {
				logger.log(Level.FINER,"SSO Id " + ssoId + " set; attempting " +	"reauthentication");
				// Try to reauthenticate using data cached by SSO.  If this fails,
				// either the original SSO logon was of DIGEST or SSL (which
				// we can't reauthenticate ourselves because there is no
				// cached username and password), or the realm denied
				// the user's reauthentication for some reason.
				// In either case we have to prompt the user for a logon */
				if (reauthenticateFromSSO(ssoId, request))
				{
					return true;
				}
			}

			// Have we authenticated this user before but have caching disabled?
			if (!cache)
			{
				session = request.getSessionInternal( true);
				logger.log(Level.FINER,"Checking for reauthenticate in session " + session);
				String username =(String) session.getNote(Constants.SESS_USERNAME_NOTE);
				String password =(String) session.getNote(Constants.SESS_PASSWORD_NOTE);
				if ((username != null) && (password != null))
				{
					logger.log(Level.FINER,"Reauthenticating username '" + username + "'");
					setRequestInThreadLocal(hreq);
					if(!principalCreated)
					{
						principal = context.getRealm().authenticate(username, password);
					}
					else
					{
						principal=twoFactorNtlmCachedPrincipal;
					}
					if (principal != null) {
						session.setNote(Constants.FORM_PRINCIPAL_NOTE, principal);
						if (!matchRequest(request)) {
							register(request, response, principal,	Constants.FORM_METHOD,	username, password);
							return (true);
						}
					}
					logger.log(Level.FINER,"Reauthentication failed, proceed normally");
				}
			}

			// Is this the re-submit of the original request URI after successful
			// authentication?  If so, forward the *original* request instead.
			if (matchRequest(request)) {
				session = request.getSessionInternal( true);
				logger.log(Level.FINER,"Restore request from session '" + session.getId() + "'");
				principal = (Principal)session.getNote(Constants.FORM_PRINCIPAL_NOTE);
				register(request, response, principal, Constants.FORM_METHOD,	(String) session.getNote(Constants.SESS_USERNAME_NOTE), (String) session.getNote(Constants.SESS_PASSWORD_NOTE));
				ssoId = (String) session.getNote(Constants.REQ_SSOID_NOTE);
				ssoId = (String) request.getNote(Constants.REQ_SSOID_NOTE);
				if(ssoId != null) {
					hreq.getSession().setAttribute("JSESSIONIDSSO", ssoId);
					associate(ssoId, session);
					Object credential = hreq.getSession().getAttribute("com.adventnet.authentication.Credential");
					associateSsoSession(ssoId, credential);
				}
				if (restoreRequest(request, session))
				{
					logger.log(Level.FINER,"Proceed to restored request");
					return (true);
				}
				else
				{
					logger.log(Level.FINER,"Restore of original request failed");
					hres.sendError(HttpServletResponse.SC_BAD_REQUEST);
					return (false);
				}
			}

			// Acquire references to objects we will need to evaluate
			MessageBytes uriMB = MessageBytes.newInstance();
			CharChunk uriCC = uriMB.getCharChunk();
			uriCC.setLimit(-1);
			String contextPath = hreq.getContextPath();
			String requestURI = request.getDecodedRequestURI();
			response.setContext(request.getContext());

			// Is this the action request from the login page?
			boolean loginAction =
				requestURI.startsWith(contextPath) &&
				requestURI.endsWith(Constants.FORM_ACTION);

			// No -- Save this request and redirect to the form login page
			if (!loginAction) {
				session = request.getSessionInternal( true);
				logger.log(Level.FINER,"Save request in session '" + session.getId() + "'");
				saveRequest(request, session);
				RequestDispatcher disp =context.getServletContext().getRequestDispatcher(config.getLoginPage());
				try
				{
					disp.forward(hreq, hres);
					response.finishResponse();
				}
				catch (Throwable t)
				{
					logger.log(Level.WARNING,"Unexpected error forwarding to login page", t);
				}
				return (false);
			}

			// Yes -- Validate the specified credentials and redirect
			// to the error page if they are not correct
			Realm realm = context.getRealm();
			boolean committed=false;
			if("UTF-8".equals(hreq.getSession().getServletContext().getInitParameter("PARAMETER-ENCODING")))
			{
				hreq.setCharacterEncoding("UTF-8");
				hres.setCharacterEncoding("UTF-8");
				hres.setContentType("text/html; charset=utf-8; encoding=UTF-8");
			}
			String username = hreq.getParameter(Constants.FORM_USERNAME);
			String password = hreq.getParameter(Constants.FORM_PASSWORD);
			logger.log(Level.FINER,"Authenticating username '" + username + "'");
			setRequestInThreadLocal(hreq);

			if(principal==null && "redirect".equals(hreq.getParameter("ntlm")))
			{
				twoFactorNtlmCachedPrincipal=(Principal)hreq.getSession().getAttribute("2FactorPrincipalRedirect");
				principal=twoFactorNtlmCachedPrincipal;
				principalCreated=true;
			}
			if("true".equals(hreq.getParameter("ntlm")))
			{
				try{
					boolean ntlmauth=AuthUtil.tryNtlmAuthentication(hreq,hres);
					if("true".equals(hreq.getAttribute("NTLMCOMM")))
					{
						hreq.getSession().setAttribute("NTLMAUTH", "failed");
						return false;
					}
					if(ntlmauth==false)
					{
						principalCreated=false;
					}
					else
					{
						if("domain.backslash.username".equals(System.getProperty("ADUserNameSyntax")))
						{
							principal=PAM.login(hreq.getAttribute("domainName")+"\\"+hreq.getAttribute("userName"),"System",hreq);
						}
						else
						{
							principal=PAM.login(hreq.getAttribute("userName").toString(),"System",hreq);
						}
						request.setUserPrincipal(principal);
						request.getSessionInternal(false).setNote(Constants.FORM_PRINCIPAL_NOTE, principal);
						request.setAuthType("NTLM");
						principalCreated=true;
					}
				}
				catch(Exception exp)
				{
					exp.printStackTrace();
				}

			}
			else if("true".equals(hreq.getParameter("ntlmv2")))
			{
				try{

					String nv2UserName=hreq.getSession().getAttribute("NtlmUserName").toString();
					String nv2DomainName=hreq.getSession().getAttribute("NtlmDomainName").toString();
					hreq.setAttribute("domainName", nv2DomainName);
					if("domain.backslash.username".equals(System.getProperty("ADUserNameSyntax")))
					{
						principal=PAM.login(nv2DomainName+"\\"+nv2UserName,"System",hreq);
					}
					else
					{
						principal=PAM.login(nv2UserName,"System",hreq);
					}
					request.setUserPrincipal(principal);
					request.getSessionInternal(false).setNote(Constants.FORM_PRINCIPAL_NOTE, principal);
					request.setAuthType("NTLM");
					principalCreated=true;

				}
				catch(Exception exp)
				{
					hreq.setAttribute("NTLMAUTHEN","false");
					hreq.getSession().setAttribute("NTLMFAILED", "true");
					hreq.getSession().setAttribute("nv2login","false");
					exp.printStackTrace();
				}

			}
			else if("true".equals(hreq.getParameter("smartcard")))
			{
				try
				{
					String smartcardusername = (String) hreq.getSession().getAttribute("SmartCardUserName");
					if(smartcardusername!=null)
					{
						principal=PAM.login(smartcardusername,"System",hreq);
						request.setUserPrincipal(principal);
						request.getSessionInternal(false).setNote(Constants.FORM_PRINCIPAL_NOTE, principal);
						request.setAuthType("SMARTCARD");
						principalCreated=true;
					}
				}
				catch(Exception e)
				{
					hreq.getSession().setAttribute("SMARTCARDFAILED", "true");
					hreq.getSession().setAttribute("SMARTCARDLOGIN", "failed");
					e.printStackTrace();
				}
			}
			else
			{
				if(!principalCreated)
				{
					principal = realm.authenticate(username, password);
				}
				else
				{
					principal=twoFactorNtlmCachedPrincipal;
				}

			}
			if (principal == null) {
				RequestDispatcher disp =
					context.getServletContext().getRequestDispatcher
					(config.getErrorPage());
				try {
					if(!committed)
					{
						if("false".equals(hreq.getAttribute("NTLMAUTHEN")))
						{
							String url=config.getLoginPage();
							if(hreq.getSession().getAttribute("nv2loginpage")!=null)
							{
								url=hreq.getSession().getAttribute("nv2loginpage").toString();
							}
							//	hreq.getSession().setAttribute("nv2login", "false");
							handleNtlmRedirect(hres,url , false);
						}
						else
						{
							disp.forward(hreq, hres);
						}
					}
				} catch (Throwable t) {
					logger.log(Level.WARNING,"Unexpected error forwarding to error page", t);
				}
				return (false);
			}

			logger.log(Level.FINER,"Authentication of '" + username + "' was successful");
			if (session == null)
			{
				session = request.getSessionInternal( false);
			}
			if (session == null) {
				logger.log(Level.FINER,"User took so long to log on the session expired");
				hres.sendError(HttpServletResponse.SC_REQUEST_TIMEOUT,sm.getString("authenticator.sessionExpired"));
				return (false);
			}

			// Save the authenticated Principal in our session
			session.setNote(Constants.FORM_PRINCIPAL_NOTE, principal);

			// If we are not caching, save the username and password as well
			if (!cache) {
				session.setNote(Constants.SESS_USERNAME_NOTE, username);
				session.setNote(Constants.SESS_PASSWORD_NOTE, password);
			}

			// Redirect the user to the original request URI (which will cause
			// the original request to be restored)
			requestURI = savedRequestURL(session);
			logger.log(Level.FINEST, "server : {0}, context path : {1}, local addr : {2} local port : {3} , " +
					"reqUri : {4}, login page : {5}, error page : {6}",  new Object [] {hreq.getServerName(),
					hreq.getContextPath(), hreq.getLocalAddr(), new Integer(hreq.getLocalPort()), requestURI,
					config.getLoginPage(), config.getErrorPage()});
			logger.log(Level.FINEST, "original requestURI : {0}", requestURI);
			if( (requestURI != null) && ( (requestURI.indexOf("login.jsp") > 1) || (requestURI.indexOf("logon.jsp") > 1) ||
						(requestURI.indexOf("login.do") > 1) || (requestURI.indexOf("logon.do") > 1)) ) {
				logger.log(Level.FINE, "since reqURI is index of login forwarding to home");
				requestURI = hreq.getContextPath();
			}
			requestURI = (requestURI == null) ? hreq.getContextPath() : requestURI;
			logger.log(Level.FINEST, "requestURI to be forwarded to : {0}", requestURI);
			if (requestURI == null)
			{
				hres.sendError(HttpServletResponse.SC_BAD_REQUEST, sm.getString("authenticator.formlogin"));
			}
			else
			{
				hreq.getSession().setAttribute("2FactorPrincipal", null);
				if(hreq.getAttribute("NTLMAUTHEN") == null)
				{
					hres.sendRedirect(hres.encodeRedirectURL(requestURI));
				}
				else if(hreq.getAttribute("NTLMAUTHEN").equals("false"))
				{
					handleNtlmRedirect(hres,requestURI,false);
					return true;
				}
				else
				{
					handleNtlmRedirect(hres,requestURI,true);
					return true;
				}
			}
			return (false);
		}
	public static void handleNtlmRedirect(HttpServletResponse hres,String requestURI,boolean success)
	{
		String url = hres.encodeRedirectURL(requestURI);
		try
		{
			hres.setStatus(401);
			PrintWriter out = hres.getWriter();
			out.println("<html>");
			out.println("<head>");
			if(success)
			{
				out.println("<title>NTLM Success you are authenticated.</title>");
			}
			else
			{
				out.println("<title>NTLM Failure you are being redirected.</title>");
			}
			out.println("<script language=\"javascript\">");
			out.println("<!--");
			out.println((new StringBuilder()).append("location.replace(\"").append(url).append("\");").toString());
			out.println("//-->");
			out.println("</script>");
			out.println("<noscript>");
			out.println((new StringBuilder()).append("<meta http-equiv=\"Refresh\" content=\"0; URL=").append(url.toString()).append("\">").toString());
			out.println("</noscript>");
			out.println("</head>");
			out.println("<body>");
			if(success)
			{
				out.println((new StringBuilder()).append("You were logged in successfully.  Click <a href=\"").append(url.toString()).append("\">here</a> to continue.").toString());
			}
			else
			{
				out.println((new StringBuilder()).append("NTLM failed.  Click <a href=\"").append(url.toString()).append("\">here</a> to continue.").toString());

			}
			out.println("</body>");
			out.println("</html>");
			hres.flushBuffer();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

	}

	// Changes to set Request in thread local and to associate ssoId
	private java.lang.reflect.Method associateSsoSessionMethod = null;
	private java.lang.reflect.Method setRequestInThreadLocalMethod = null;

	private void createInstance(){
		Class authDbUtilClass = null;
		Class customJassRealmClass = null;
		try{
			if(associateSsoSessionMethod == null) {
				authDbUtilClass = Class.forName("com.adventnet.authentication.util.AuthDBUtil");
				Class[] params = new Class[2];
				params[0] = Class.forName("java.lang.String");
				params[1] = Class.forName("com.adventnet.authentication.Credential");
				associateSsoSessionMethod = authDbUtilClass.getDeclaredMethod("associateSsoSession",params);
			}
			if(setRequestInThreadLocalMethod == null) {
				customJassRealmClass = Class.forName("com.adventnet.authentication.realm.CustomJAASRealm");
				Class[] params = new Class[1];
				params[0] = Class.forName("javax.servlet.http.HttpServletRequest");
				setRequestInThreadLocalMethod = customJassRealmClass.getDeclaredMethod("setRequestInThreadLocal", params);
			}
		}
		catch(Exception e) {
			logger.log(Level.SEVERE, "Exception occured while creating method instance for associateSsoSession and setRequestInThreadLocal", e);
		}
	}

	private void setRequestInThreadLocal(HttpServletRequest httpServletRequest) {
		if(setRequestInThreadLocalMethod == null) {
			createInstance();
		}
		try {
			logger.log(Level.FINEST, "invoking method setRequestInThreadLocal with params");
			Object[] args = new Object[1];
			args[0] = httpServletRequest;
			setRequestInThreadLocalMethod.invoke(null, args);
		}
		catch(Exception e) {
			logger.log(Level.SEVERE, "Exception occured while setting request in threadLocal : ", e);
		}
	}

	private void associateSsoSession(String key, Object value) {
		logger.log(Level.FINEST, "associateCredential invoked with Key : {0} and value : {1}", new Object [] {key, value});
		if(associateSsoSessionMethod == null) {
			createInstance();
		}
		try{
			Object[] args = new Object[2];
			args[0] = key;
			args[1] = value;
			logger.log(Level.FINEST, "invoking method addUserCredential with params");
			associateSsoSessionMethod.invoke(null, args);
		}
		catch(Exception e){
			logger.log(Level.SEVERE, "Exception caught while tring to associateSsoId : {0}", e);
		}
	}

	// ------------------------------------------------------ Protected Methods


	/**
	 * Does this request match the saved one (so that it must be the redirect
	 * we signalled after successful authentication?
	 *
	 * @param request The request to be verified
	 */
	protected boolean matchRequest(Request request) {

		// Has a session been created?
		Session session = request.getSessionInternal( false);
		if (session == null)
		{
			return (false);
		}

		// Is there a saved request?
		SavedRequest sreq = (SavedRequest)
			session.getNote(Constants.FORM_REQUEST_NOTE);
		if (sreq == null)
		{
			return (false);
		}
		// Is there a saved principal?
		if (session.getNote(Constants.FORM_PRINCIPAL_NOTE) == null)
		{
			return (false);
		}

		// Does the request URI match?
		HttpServletRequest hreq = (HttpServletRequest) request.getRequest();
		String requestURI = hreq.getRequestURI();
		if (requestURI == null)
		{
			return (false);
		}
		return (requestURI.equals(sreq.getRequestURI()));

	}


	/**
	 * Restore the original request from information stored in our session.
	 * If the original request is no longer present (because the session
	 * timed out), return <code>false</code>; otherwise, return
	 * <code>true</code>.
	 *
	 * @param request The request to be restored
	 * @param session The session containing the saved information
	 */
	protected boolean restoreRequest(Request request, Session session) {

		// Retrieve and remove the SavedRequest object from our session
		SavedRequest saved = (SavedRequest)
			session.getNote(Constants.FORM_REQUEST_NOTE);
		session.removeNote(Constants.FORM_REQUEST_NOTE);
		session.removeNote(Constants.FORM_PRINCIPAL_NOTE);
		if (saved == null)
		{
			return (false);
		}
		// Modify our current request to reflect the original one
		request.clearCookies();
		Iterator cookies = saved.getCookies();
		while (cookies.hasNext()) {
			request.addCookie((Cookie) cookies.next());
		}
		request.clearHeaders();
		Iterator names = saved.getHeaderNames();
		while (names.hasNext()) {
			String name = (String) names.next();
			Iterator values = saved.getHeaderValues(name);
			while (values.hasNext()) {
				request.addHeader(name, (String) values.next());
			}
		}
		request.clearLocales();
		Iterator locales = saved.getLocales();
		while (locales.hasNext()) {
			request.addLocale((Locale) locales.next());
		}
		request.clearParameters();
		if ("POST".equalsIgnoreCase(saved.getMethod())) {
			Iterator paramNames = saved.getParameterNames();
			while (paramNames.hasNext()) {
				String paramName = (String) paramNames.next();
				String paramValues[] =
					saved.getParameterValues(paramName);
				request.addParameter(paramName, paramValues);
			}
		}
		request.setMethod(saved.getMethod());
		request.setQueryString(saved.getQueryString());
		request.setRequestURI(saved.getRequestURI());
		return (true);

	}


	/**
	 * Save the original request information into our session.
	 *
	 * @param request The request to be saved
	 * @param session The session to contain the saved information
	 */
	protected void saveRequest(Request request, Session session) {

		// Create and populate a SavedRequest object for this request
		HttpServletRequest hreq = (HttpServletRequest) request.getRequest();
		SavedRequest saved = new SavedRequest();
		Cookie cookies[] = hreq.getCookies();
		if (cookies != null)
		{
			for (int i = 0; i < cookies.length; i++)
			{
				saved.addCookie(cookies[i]);
			}
		}
		Enumeration names = hreq.getHeaderNames();
		while (names.hasMoreElements()) {
			String name = (String) names.nextElement();
			Enumeration values = hreq.getHeaders(name);
			while (values.hasMoreElements()) {
				String value = (String) values.nextElement();
				saved.addHeader(name, value);
			}
		}
		Enumeration locales = hreq.getLocales();
		while (locales.hasMoreElements()) {
			Locale locale = (Locale) locales.nextElement();
			saved.addLocale(locale);
		}
		Map parameters = hreq.getParameterMap();
		Iterator paramNames = parameters.keySet().iterator();
		while (paramNames.hasNext()) {
			String paramName = (String) paramNames.next();
			String paramValues[] = (String[]) parameters.get(paramName);
			saved.addParameter(paramName, paramValues);
		}
		saved.setMethod(hreq.getMethod());
		saved.setQueryString(hreq.getQueryString());
		saved.setRequestURI(hreq.getRequestURI());

		// Stash the SavedRequest in our session for later use
		session.setNote(Constants.FORM_REQUEST_NOTE, saved);

	}


	/**
	 * Return the request URI (with the corresponding query string, if any)
	 * from the saved request so that we can redirect to it.
	 *
	 * @param session Our current session
	 */
	protected String savedRequestURL(Session session) {

		SavedRequest saved =
			(SavedRequest) session.getNote(Constants.FORM_REQUEST_NOTE);
		if (saved == null)
		{
			return (null);
		}
		StringBuffer sb = new StringBuffer(saved.getRequestURI());
		if (saved.getQueryString() != null) {
			sb.append('?');
			sb.append(saved.getQueryString());
		}
		return (sb.toString());

	}





	public boolean validateTwoFactorAuth(String loginName,HttpServletRequest request)
	{
		Long userId=null;
		try{
			String domainName=request.getParameter(PAM.DOMAINNAME);
			if(domainName!=null)
			{
				if("domain.backslash.username".equals(System.getProperty("ADUserNameSyntax")) && !loginName.contains("\\"))
				{
					userId=AuthUtil.getUserId(domainName+"\\"+loginName,domainName);
				}
				else
				{
					userId=AuthUtil.getUserId(loginName,domainName);
				}
			}
			else
			{
				userId=AuthUtil.getUserId(request.getParameter(PAM.USERNAME));
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			userId=null;
		}

		try
		{

			if(System.getProperty("2factor.auth")==null && !AuthUtil.isTwofactorLoginEnabled(userId))
			{
				return true;
			}
		}
		catch(Exception e)
		{
			logger.log(Level.SEVERE,"No details for AaaUserTwoFactorDetails table for user "+loginName);
			return true;
		}
		boolean ret=false;
		try
		{
			TwoFactorAuth  userAuthImpl=(TwoFactorAuth)(AuthUtil.getTwoFactorImpl(userId));
			ret=	userAuthImpl.validate(userId,request);

		}
		catch(Exception e)
		{
			logger.log(Level.SEVERE,"Exception while instantiating Two Factor implementation class or No details for AaaUserTwoFactorDetails table for user "+loginName);
			return ret;
		}
		return ret;

	}
}